function compensator = compensator_step(compensator, t_vals, y_vals, u, apply_fb);

[n, m] = size(compensator.B);

ode_func = @(t, z) ode_func_interpolate(t, z, ...
          t_vals, y_vals, u, compensator, apply_fb, true);
[TOUT, XOUT] = ode45(ode_func, t_vals, compensator.z0);
clear ode_func;
compensator.z0 = XOUT(end, :)';

ode_func = @(t, z) ode_func_interpolate(t, z, ...
          [t_vals(end), t_vals(end)+compensator.h_t], ...
          [y_vals(:, end), y_vals(:, end)], u, compensator, apply_fb, false);
[TOUT, XOUT] = ode45(ode_func, t_vals+compensator.h_t, compensator.z0);
clear ode_func;
ZOUT = compensator.F * XOUT';

compensator.feedback = @(t) feedback_func_interpolate(t, TOUT, ZOUT);


%%%%%%%%%%%%%%%
function res = ode_func_interpolate(t, z, t_vals, y_vals, u, compensator, apply_fb, real_feedback)

assert(t >= t_vals(1) && t <= t_vals(end));

[n, m] = size(compensator.B);

index = 1;
while index<(length(t_vals)-1) && t_vals(index+1)<t
  index = index+1;
end

t_size = t_vals(index)-t_vals(index+1);
the_y = (t-t_vals(index+1))/t_size*y_vals(:, index) + ...
        (t_vals(index)-t)/t_size*y_vals(:, index+1);

if( apply_fb && real_feedback )
  res = compensator.A*z + compensator.L*(compensator.C*z) + compensator.B*(compensator.feedback(t)) + ...
        compensator.B*u - compensator.L*the_y;
elseif( apply_fb && ~real_feedback )
  res = compensator.A*z + compensator.L*(compensator.C*z) + compensator.B*(compensator.F*z) + ...
        compensator.B*u - compensator.L*the_y;
else
  res = compensator.A*z + compensator.L*(compensator.C*z) + ...
      + compensator.B*u - compensator.L*the_y;
end

%%%%%%%%%%%%%%%
function the_f = feedback_func_interpolate(t, t_vals, f_vals)

assert(t >= t_vals(1) && t <= t_vals(end));

index = 1;
while index<(length(t_vals)-1) && t_vals(index+1)<t
  index = index+1;
end

t_size = t_vals(index)-t_vals(index+1);
the_f = (t-t_vals(index+1))/t_size*f_vals(:, index) + ...
        (t_vals(index)-t)/t_size*f_vals(:, index+1);

