function simulate_realistic(A, B, C, D, h_t, order_t)

[n, m] = size(B);

plot_n = n/order_t;

assert(round(plot_n) == plot_n, ...
       'the dimension of the state-space could not be divided by order_t');

hSpace = 1/(plot_n+1);
grid = linspace(hSpace, 1-hSpace, plot_n)';

w_func = @(pos, middle) (sign(pos+0.1-middle).*sign(middle-pos+0.1)+1)/5;
w_func = @(pos, middle) exp(-(pos-middle).^2*20)/30;

x0 = zeros(n, 1);

MAX_X =  50.0;
MIN_X = -50.0;

% initialize callbacks
clf;

wideness_side_view = 3;
wideness_total = 2*wideness_side_view + 1;

%%%%%%%%%%%%%%%%%%
% LEFT: START
%%%%%%%%%%%%%%%%%%

subplot(1, wideness_total, 1:wideness_side_view);

plot([0, 1], [MAX_X, MIN_X], 'kx', 'MarkerSize', 0.001);
hold on;

plot(grid, B((order_t-1)*plot_n + (1:plot_n), :)', 'g-');
plot(grid, C(:, 1:plot_n ), 'r-');

plot_handle_left = ...
      plot(grid, x0(1:plot_n), 'b-', 'YDataSource', 'act_x');

title('press any keyboard key to quit');
hold off;

fig_handle = gcf;
set(fig_handle, 'WindowButtonDownFcn', @mouse_down);
set(fig_handle, 'WindowButtonUpFcn', @mouse_up);
set(fig_handle, 'WindowButtonMotionFcn', @mouse_move);
set(fig_handle, 'KeyPressFcn',@keyPressCallback);

%%%%%%%%%%%%%%%%%%
% LEFT: END
%%%%%%%%%%%%%%%%%%

ah_left = gca;

%%%%%%%%%%%%%%%%%%
% MIDDLE: START
%%%%%%%%%%%%%%%%%%

subplot(1, wideness_total, wideness_side_view+1);
plot([0, 1], [0, 0], 'k-');
hold on;
plot([0.5, 0.5], sum(sum(abs(B)))*[MIN_X, MAX_X], 'k-');

output = zeros(m, 1);
comp_in = zeros(m, 1);
for i=1:m
plot_handle_middle{i} = ...
      plot( [0, 1], [output(i), output(i)]', 'r-', 'YDataSource', ...
      ['[output(',num2str(i),'), output(',num2str(i),')]'] );
plot_handle_middle_in{i} = ...
      plot( [0, 1], [comp_in(i), comp_in(i)]', 'g-', 'YDataSource', ...
      ['[comp_in(',num2str(i),'), comp_in(',num2str(i),')]'] );
end
hold off;

%%%%%%%%%%%%%%%%%%
% MIDDLE: END
%%%%%%%%%%%%%%%%%%

ah_middle = gca;

%%%%%%%%%%%%%%%%%%
% RIGHT: START
%%%%%%%%%%%%%%%%%%

subplot(1, wideness_total, wideness_side_view+2:wideness_total);

plot([0, 1], [MAX_X, MIN_X], 'kx', 'MarkerSize', 0.001);
hold on;

z0 = zeros(n, 1);
plot_handle_right = ...
      plot(grid, z0(1:plot_n), 'b-', 'YDataSource', 'act_z');

hold off;

%%%%%%%%%%%%%%%%%%
% RIGHT: END
%%%%%%%%%%%%%%%%%%

uicontrol('Style', 'popup', ...
          'String', 'state|observer', ...
          'Position', [20, 0, 100, 30], ...
          'Callback', @popup_select);
uicontrol('Style', 'togglebutton', ...
          'String', 'Feedback', ...
          'Position', [120, 0, 100, 30], ...
          'Callback', @btn_feedback);
uicontrol('Style', 'togglebutton', ...
          'String', 'Observe', ...
          'Position', [220, 0, 100, 30], ...
          'Callback', @btn_observe);
uicontrol('Style', 'togglebutton', ...
          'String', 'more Realistic', ...
          'Position', [380, 0, 100, 30], ...
          'Callback', @btn_realistic);
uicontrol('Style', 'togglebutton', ...
          'String', 'Quit', ...
          'Position', [480, 0, 80, 30], ...
          'Callback', @btn_quit);

w_fact = 0.0;
w_middle = 0.0;

terminate = false;

input = zeros(m, 1);
t = 0.0;

compensator = compensator_init(A, B, C, D, h_t);

apply_fb = false;
do_observe = false;
fb_type = 'state';
realistic = false;

LC = compensator.L*C;
BF = B*compensator.F;
A_BF = A + BF;
A_LC = A + LC;
A_LC_BF = A_LC + BF;

BB = [B; B];

while( ~terminate )

  w_disc = zeros(size(x0));
  if( w_fact~=0.0 )
    w_disc( (order_t-1)*plot_n + (1:plot_n) ) = w_fact*w_func(grid, w_middle);
  end

  if( realistic )

    if( apply_fb )
      ode_func = @(t, y) A*y + B*compensator.feedback(t) + B*input + w_disc;
    else
      ode_func = @(t, y) A*y + B*input + w_disc;
    end
    [TOUT, XOUT] = ode45(ode_func, [t, t+h_t], x0); XOUT = XOUT';
    x0 = XOUT(:, end);
    if( do_observe )
       OUTPUT = C*XOUT + D*input*ones(1, size(XOUT, 2));
    else
       OUTPUT = zeros(size(C, 1), size(XOUT, 2));
    end
    output = C*XOUT(:, end) + D*input;

    compensator = compensator_step(compensator, TOUT, OUTPUT, input, apply_fb);
    z0 = compensator.z0;
    comp_in = compensator.F * z0;

  else

    if( apply_fb && strcmp(fb_type, 'state') )
      ode_func = @(t, y) [A_BF, zeros(n); zeros(n), A]*y + BB*input + [w_disc; zeros(n, 1)];
    else
      MATRIX = [A, apply_fb*BF; -do_observe*LC, A+do_observe*LC+apply_fb*BF];
      ode_func = @(t, y) MATRIX*y + BB*input + [w_disc; zeros(n, 1)];
    end

    [TOUT, XOUT] = ode45(ode_func, [t, t+h_t], [x0; z0]); XOUT = XOUT';
    x0 = XOUT(  1:  n, end);
    z0 = XOUT(n+1:2*n, end);

    output = C*x0 + D*input;

    compensator.z0 = z0;

    if( apply_fb && strcmp(fb_type, 'state') )
      comp_in = compensator.F * x0;
    else
      comp_in = compensator.F * z0;
    end

  end


  %norm(x0-compensator.z0)

  % plot actual state
  act_x = x0(1:plot_n);
  act_z = z0(1:plot_n);
  refreshdata(plot_handle_left, 'caller');
  refreshdata(plot_handle_middle, 'caller');
  refreshdata(plot_handle_middle_in, 'caller');
  refreshdata(plot_handle_right, 'caller');
  drawnow;

  t = t+h_t;
end

close(fig_handle);

   %
   % Callback functions
   %
   function mouse_down(src, data)
      pnt_left   = get(ah_left, 'CurrentPoint');
      pnt_middle = get(ah_middle, 'CurrentPoint');

      w_middle = pnt_left(1, 1);

      if( pnt_left(1, 1) >= 0.0 && pnt_left(1, 1) <= 1.0 )
        switch( get(fig_handle, 'SelectionType') )
           case 'normal' % left mouse button
              w_fact = w_fact + 3.0;
           case 'alt' % left mouse button
              w_fact = w_fact - 3.0;
           case 'extend' % left mouse button
        end
      elseif( pnt_middle(1, 1) >= 0.0 && pnt_middle(1, 1) <= 1.0 )
        switch( get(fig_handle, 'SelectionType') )
           case 'normal' % left mouse button
              input = input + 0.1;
           case 'alt' % left mouse button
              input = input - 0.1;
           case 'extend' % left mouse button
        end
      end
   end

   function mouse_up(src, data)
      w_fact = 0.0;
      input = zeros(m, 1);
   end

   function mouse_move(src, data)
      pnt = get(ah_left, 'CurrentPoint');
      w_middle = pnt(1, 1);
   end

   function keyPressCallback(src, data)
      terminate = true;
   end

   function popup_select(hObj, event)
      if( get(hObj, 'Value') == 1 )
         fb_type = 'state';
      else
         fb_type = 'observer';
      end
   end

   function btn_feedback(hObj, event)
      apply_fb = get(hObj, 'Value');
   end

   function btn_observe(hObj, event)
      do_observe = get(hObj, 'Value');
   end

   function btn_quit(hObj, event)
      terminate = true;
   end

   function btn_realistic(hObj, event)
      realistic = get(hObj, 'Value');
   end
end

